unit FGuiServer;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, System.Types, Vcl.Graphics,syncobjs, Vcl.Controls, Vcl.Forms,
  Vcl.Dialogs, Vcl.Buttons, Vcl.StdCtrls, Idcontext, IdGlobal, IdBaseComponent,
  IdComponent, IdCustomTCPServer, IdTCPServer, Vcl.ExtCtrls, Vcl.ComCtrls,
  Vcl.AppEvnts;

type
  TFServer = class(TForm)
    ButtonLogIn: TSpeedButton;
    Label1: TLabel;
    Edit1: TEdit;
    IdTCPServer1: TIdTCPServer;
    Edit2: TEdit;
    Timer1: TTimer;
    LblClientcount: TLabel;
    EdtClientCount: TEdit;
    RichEdit1: TRichEdit;
    ApplicationEvents1: TApplicationEvents;
    procedure ButtonLogInClick(Sender: TObject);
    procedure IdTCPServer1Execute(AContext: TIdContext);
    procedure IdTCPServer1Status(ASender: TObject; const AStatus: TIdStatus;
      const AStatusText: string);
    procedure IdTCPServer1Exception(AContext: TIdContext;
      AException: Exception);
    procedure IdTCPServer1Connect(AContext: TIdContext);
    procedure Timer1Timer(Sender: TObject);
    procedure Edit2KeyPress(Sender: TObject; var Key: Char);
    procedure Sendtext(s: string; SkipMePlease: TIdContext);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
  private
    lock:TCriticalsection;
    sllog:TStringlist;
    procedure ColLog(aEdit: TRichEdit; aMsg: string; aColor: TColor; UpdateNow :Boolean = False;
      Limitlines: boolean = True);
  public
    procedure Log(Msg: string; col:Tcolor);
  end;

var
  FServer: TFServer;

implementation

{$R *.dfm}

procedure TFServer.ColLog(aEdit:TRichEdit; aMsg:string; aColor:TColor;
                  UpdateNow:boolean=false; Limitlines:boolean=true);
var h,fh:integer;
begin
  for h:=1 to length(aMsg) do if not (charinset(aMsg[h],[#32..#127])) then aMsg[h]:='|';
  aEdit.CaretPos:=point(0,aEdit.lines.count);
  aEdit.SelAttributes.Color:=aColor;  aEdit.Lines.add(aMsg);
  h:=aEdit.ClientHeight; if h<26 then h:=26; fh:= abs(aEdit.Font.Height)+3;
  if limitlines then while aEdit.lines.count >= h div fh {14} do aEdit.Lines.Delete(0);
  if UpdateNow  then aEdit.Update;
end;

procedure TFServer.ButtonLogInClick(Sender: TObject);
begin
  IdTCPServer1.DefaultPort := StrToIntdef(Edit1.Text, 888);
  Edit1.Text := IntTostr(IdtcpServer1.DefaultPort);
  if IdTCPServer1.Active then  IdTCPServer1.Active := False;
  Try
    IdTCPServer1.Active := ButtonLogIn.Down;
  Except
    on e: exception do log(E.message, clred);// (log mag vcl click)
  End;
  if IdTCPServer1.Active then log('Server activated', clgreen)
  else
     log(' Server NOT activated', clBlue);
end;

procedure TFServer.Edit2KeyPress(Sender: TObject; var Key: Char);
begin
  sendtext(key,Nil);  // only 1 char, but string is also allowed
  if Key = #13 then  // carriagereturn
  begin
    edit2.text := '';
    Key := #0;   // overcomes warning sound
  end;
end;

procedure TFServer.FormCreate(Sender: TObject);
begin
  lock:=TCriticalsection.create();
  slLog:=TStringlist.Create();
end;

procedure TFServer.FormDestroy(Sender: TObject);
begin
  lock.Free;
  slLog.Free;
end;

procedure TFServer.IdTCPServer1Connect(AContext: TIdContext);
begin
  log(AContext.Connection.Socket.BoundIP + 'Connected',ClBlue)  ;
  AContext.Connection.Socket.WriteLn('Good Meurning');
end;

procedure TFServer.IdTCPServer1Exception(AContext: TIdContext;
  AException: Exception);
begin
  log(AContext.Connection.Socket.BoundIP +' '+ AException.Message, clred);
end;

procedure TFServer.IdTCPServer1Execute(AContext: TIdContext);
Var B: Char;
begin
    Try
      if AContext.Connection.Connected  then
      begin
        B:= Char(AContext.Connection.IOHandler.ReadByte());
        sendtext(B,acontext);
      end;
    Except
      on e: exception do log(E.message, clred);
    End;
end;

procedure TFServer.IdTCPServer1Status(ASender: TObject;
  const AStatus: TIdStatus; const AStatusText: string);
begin
  log(AStatusText,clgreen);
end;

procedure TFServer.ApplicationEvents1Idle(Sender: TObject; var Done: Boolean);
begin  // running from vcl thread after all activity is done.
       // it is safe to copy content of slLog to edit box here.
  lock.Enter;
  try
    while slLog.Count>0 do begin
      collog(RichEdit1,slLog[0],clBlue);
      slLog.Delete(0);
    end;
  finally
    lock.Leave;
  end;
end;

procedure TFServer.Log(Msg: string; col:Tcolor);
begin  // can be called from any thread (or VCL), no acces to VCL controlls directly.
  lock.enter;       // lock stringlist from all other threads
  try
    slLog.Add(msg); // add logged message to stringlist
  finally
    lock.Leave;     // allow other threads to access stringlist again.
  end;
end;

procedure TFServer.SendText(s: string; SkipMePlease: TIdContext);
var List : Tlist;
    N    : integer;
    ctx  : TidContext;
begin
  List:= IdTcpServer1.Contexts.LockList;// Must be unlocked in extra try finally
  // sever has separate thread for ech cneected client, so it must be locked and unlocked
  // contains all client threads and will noet change until unlock list
  // (threadsafe list)
  Try
    for N:=0 to list.Count -1 do
    begin
      CTX := TIdContext(list[N]);   //for all connected clients
      if CTX <> SkipMePlease then   //if client is not me
      try
        if ctx.Connection.Connected then  // and client is still connected send string
          ctx.Connection.Socket.Write(S);
      Except
         on e: exception do log(E.message, clred);
      End;
    end;
  Finally
    IdTCPServer1.contexts.UnlockList;
  End;
end;

procedure TFServer.Timer1Timer(Sender: TObject);
begin
  EdtClientCount.Text := IntToStr(IdTCPServer1.Contexts.Count)
end;



end.
